/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.PetalsJmxApiFactory;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.launcher.api.server.PetalsServer;
import org.ow2.petals.launcher.api.server.PetalsServerFactory;
import org.ow2.petals.launcher.api.server.conf.ConfigurationProperties;
import org.ow2.petals.launcher.api.server.exception.PetalsServerException;
import org.ow2.petals.launcher.configuration.Configuration;
import org.ow2.petals.launcher.configuration.DefaultConfiguration;
import org.ow2.petals.launcher.exception.InvalidServerPropertiesFileException;
import org.ow2.petals.launcher.exception.PetalsLauncherException;
import org.ow2.petals.launcher.exception.UncompleteServerConfigurationException;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.Domain;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.Topology;

/**
 * Command Line Interface to start petals ESB server.
 * 
 * @author Sebastien Andre - EBM WebSourcing
 */
public class PetalsLauncher {

    /**
     * Footer part of the usage message
     */
    public final static String USAGE_FOOTER = "\nWhich evolution would you like on Petals? Share it! http://www.petalslink.com/feedback"
            + System.getProperty("line.separator");

    public static final String NAME = "Petals ESB Server";

    public static final String PETALS_HOME_PROPERTY_NAME = "petals.home";

    // font: smslant
    public static final String[] TITLE = new String[] {
            "     ___   ____ __   ___    __    ____   ____ ____ ___  ",
            "    / _ \\ / __// /_ / _ |  / /   / __/  / __// __// _ ) ",
            "   / ___// _/ / __// __ | / /__ _\\ \\   / _/ _\\ \\ / _  | ",
            "  /_/   /___/ \\__//_/ |_|/____//___/  /___//___//____/  ", "" };

    /**
     * The short name of the option setting the URL of the Petals container
     * configuration
     */
    protected final static String URL_SHORT_OPTION = "u";

    /**
     * The long name of the option setting the URL of the Petals container
     * configuration
     */
    private final static String URL_LONG_OPTION = "url";

    /**
     * The short name of the option printing help on command line
     */
    protected final static String HELP_SHORT_OPTION = "h";

    /**
     * The long name of the option printing help on command line
     */
    private final static String HELP_LONG_OPTION = "help";

    /**
     * The short name of the option printing the version of Petals
     */
    protected final static String VERSION_SHORT_OPTION = "V";

    /**
     * The long name of the option printing the version of Petals
     */
    private final static String VERSION_LONG_OPTION = "version";

    /**
     * The short name of the option printing the extended error messages
     * (stack-traces)
     */
    protected final static String ERROR_SHORT_OPTION = "e";

    /**
     * The long name of the option printing the extended error messages
     * (stack-traces)
     */
    private final static String ERROR_LONG_OPTION = "error";

    /**
     * The parameter to stop a remote container
     */
    protected final static String COMMAND_STOP = "stop";

    /**
     * Launch Petals ESB Server
     * 
     * @param config
     *            the ESB configuration
     * @throws PetalsLauncherException
     *             when an error occurs while starting petals
     * @throws InterruptedException
     * 
     */
    protected void launchPetalsServer(Configuration config) throws PetalsLauncherException,
            InterruptedException {
        final PetalsClassLoader petalsKernelClassLoader = new PetalsClassLoader(config);
        Thread.currentThread().setContextClassLoader(petalsKernelClassLoader);

        try {
            final PetalsServer kernelInstance = PetalsServerFactory.getInstance();
            System.out.println("Initializing server ...");
            this.setLogingSystemConfigurationFile(config);
            kernelInstance.init(config.getServerLocalProperties(), config.getTopology(),
                    config.getDataRootPath());

            System.out.println("Starting server ...");
            kernelInstance.start();

            synchronized (kernelInstance) {
                kernelInstance.wait();
            }
        } catch (final PetalsServerException e) {
            throw new PetalsLauncherException(e);
        } catch (final org.ow2.petals.launcher.api.server.exception.DuplicatedServiceException e) {
            throw new PetalsLauncherException(e);
        } catch (final org.ow2.petals.launcher.api.server.exception.MissingServiceException e) {
            throw new PetalsLauncherException(e);
        }
    }

    /**
     * <p>
     * Two logging systems are used by the container:
     * <ul>
     * <li>JDK common logging,</li>
     * <li>and, Log4J.</li>
     * </ul>
     * Each one have its configuration file. This method retrieves the
     * configuration files to configure the both logging systems.
     * </p>
     * <p>
     * The configuration file of the JDK commom logging system is retrieved from
     * following ordered item:
     * <ol>
     * <li>the system property
     * {@value ConfigurationProperties#LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME} ,
     * </li>
     * <li>the configuration property
     * {@value ConfigurationProperties#LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME}
     * of the container local configuration properties (ie.
     * <code>server.properties</code>),</li>
     * <li>the file '<code>logger.properties</code>' contained in the directory
     * containing the container local configuration properties (ie.
     * <code>server.properties</code>).</li>
     * </ol>
     * If a value is found, it is transmitted to the Petals kernel through the
     * system property
     * {@value ConfigurationProperties#LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME} .
     * </p>
     * <p>
     * The configuration file of the Log4J logging system is retrieved from the
     * location of the container local configuration (ie.
     * <code>server.properties</code>): the file <code>log4j.properties</code>
     * contained in the directory containing the container local configuration
     * properties (ie. <code>server.properties</code>).
     * </p>
     * 
     * @param config
     *            The configuration
     */
    private void setLogingSystemConfigurationFile(final Configuration config) {

        // Retrieve the configuration file of the JDK common logging system
        final String configurationFile = System
                .getProperty(ConfigurationProperties.LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME);
        if (configurationFile == null) {
            // Try to find the configuration file set in the local container
            // configuration properties
            final String configurationFilePropertyValue = config.getServerLocalProperties()
                    .getProperty(ConfigurationProperties.LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME);
            if (configurationFilePropertyValue != null) {
                System.setProperty(ConfigurationProperties.LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME,
                        configurationFilePropertyValue);
            } else if (config.getServerPropertiesUrl() != null) {
                try {
                    System.setProperty(
                            ConfigurationProperties.LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME,
                            new File(new File(config.getServerPropertiesUrl().toURI())
                                    .getParentFile(),
                                    ConfigurationProperties.DEFAULT_LOGGER_FILENAME)
                                    .getAbsolutePath());
                } catch (final URISyntaxException e) {
                    // This exception has not to occur
                }
            }
        }

        // Retrieve the configuration file of the Log4J logging system
        if (config.getServerPropertiesUrl() != null) {
            try {
                System.setProperty("log4j.configuration", new File(new File(config
                        .getServerPropertiesUrl().toURI()).getParentFile(), "log4j.properties")
                        .toURI().toURL().toString());
            } catch (final URISyntaxException e) {
                // This exception has not to occur
            } catch (final MalformedURLException e) {
                // This exception has not to occur
            }
        }
    }

    protected void stopLocalPetalsServer(final Configuration config)
            throws UncompleteServerConfigurationException, ConnectionErrorException,
            DuplicatedServiceException, MissingServiceException, PetalsAdminServiceErrorException,
            PetalsAdminDoesNotExistException {
        final Topology topology = config.getTopology();
        final String localContainerId = config.getLocalContainerId();
        final Domain domain = topology.getDomain();
        final List<Subdomain> subdomains = domain.getSubDomain();
        for (final Subdomain subdomain : subdomains) {
            final List<Container> containers = subdomain.getContainer();
            for (final Container container : containers) {
                if (container.getName().equals(localContainerId)) {
                    int rmiPort = 7700;
                    String user = "petals";
                    String pwd = "petals";
                    if (container.getJmxService() != null) {
                        if (container.getJmxService().getRmiPort() != 0) {
                            rmiPort = container.getJmxService().getRmiPort();
                        }
                    }
                    if (container.getUser() != null) {
                        user = container.getUser();
                    }
                    if (container.getPassword() != null) {
                        pwd = container.getPassword();
                    }

                    final JMXClient jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(
                            container.getHost(), rmiPort, user,
                            pwd);
                    jmxClient.getPetalsAdminServiceClient().stopContainer();
                }
            }
        }

    }

    /**
     * Print launcher version informations
     */
    protected static void printLauncherVersion() {
        StringBuilder sb = new StringBuilder(NAME).append(' ');

        sb.append(PetalsLauncher.class.getPackage().getImplementationVersion());
        sb.append('\n');
        sb.append(System.getProperty("java.runtime.name")).append(' ');
        sb.append(System.getProperty("java.runtime.version")).append('\n');
        sb.append(System.getProperty("os.name")).append(' ');
        sb.append(System.getProperty("os.version"));

        System.out.println(sb.toString());
    }

    /**
     * Print the usage message
     */
    private static void printUsageMessage() {

        String jar = PetalsLauncher.class.getProtectionDomain().getCodeSource().getLocation()
                .getFile();

        HelpFormatter formatter = new HelpFormatter();
        formatter.setWidth(USAGE_FOOTER.length());
        formatter.printHelp(NAME, "java -jar .../" + (new File(jar)).getName()
                + " [--help] [--version] [--error] [--url <container-cfg-url>] [stop]",
                PetalsLauncher.createOptions(), "");

        System.out.println(" " + COMMAND_STOP + "           Stop the local Petals ESB container");
        System.out.println("");
        System.out.println(USAGE_FOOTER);
    }

    public static Options createOptions() {
        final Options options = new Options();

        options.addOption(OptionBuilder.withLongOpt(HELP_LONG_OPTION).hasArg(false)
                .isRequired(false).withDescription("Print this help message and exit.")
                .create(HELP_SHORT_OPTION));
        options.addOption(OptionBuilder.withLongOpt(VERSION_LONG_OPTION).hasArg(false)
                .isRequired(false).withDescription("Print the version number and exit.")
                .create(VERSION_SHORT_OPTION));
        options.addOption(OptionBuilder.withLongOpt(ERROR_LONG_OPTION).hasArg(false)
                .isRequired(false)
                .withDescription("Print the extended error message (stack-trace).")
                .create(ERROR_SHORT_OPTION));
        options.addOption(OptionBuilder.withLongOpt(URL_LONG_OPTION).hasArg(true)
                .withArgName("container-cfg-url").isRequired(false)
                .withDescription("The URL of the Petals container configuration.")
                .create(URL_SHORT_OPTION));

        return options;
    }

    /**
     * The main entry point
     * 
     * @param args
     *            Arguments from the command line
     */
    public static void main(String[] args) {
        System.exit(PetalsLauncher.run(args));
    }

    /**
     * Parse options and command line arguments
     * 
     * @param args
     *            Arguments from the command line
     */
    protected static int run(String[] args) {

        int exitCode = 0;

        boolean errorEnable = false;
        final CommandLineParser parser = new PosixParser();

        try {
            final CommandLine cmd = parser.parse(PetalsLauncher.createOptions(), args);

            if (cmd.hasOption(ERROR_SHORT_OPTION)) {
                errorEnable = true;
            }

            if (cmd.hasOption(PetalsLauncher.HELP_SHORT_OPTION)) {
                PetalsLauncher.printUsageMessage();
            } else if (cmd.hasOption(PetalsLauncher.VERSION_SHORT_OPTION)) {
                PetalsLauncher.printLauncherVersion();
            } else {

                final URL serverLocalPropertiesUrl;
                if (cmd.hasOption(URL_SHORT_OPTION)) {
                    final String serverLocalProperties = cmd.getOptionValue(URL_SHORT_OPTION);
                    try {
                        serverLocalPropertiesUrl = new URL(serverLocalProperties);
                    } catch (final MalformedURLException e) {
                        throw new InvalidServerPropertiesFileException(
                                serverLocalProperties);
                    }
                } else {
                    serverLocalPropertiesUrl = null;
                }

                final Configuration config = new DefaultConfiguration(serverLocalPropertiesUrl);

                final String[] remainingArgs = cmd.getArgs();
                if (remainingArgs.length > 0) {
                    if (remainingArgs[0].equalsIgnoreCase(COMMAND_STOP)) {
                        final PetalsLauncher petalsLauncher = new PetalsLauncher();
                        petalsLauncher.stopLocalPetalsServer(config);
                    } else {
                        System.err.println("ERROR: Unknown option '" + remainingArgs[0] + "'\n");
                        PetalsLauncher.printUsageMessage();
                        exitCode = 1;
                    }
                } else {
                    // ascii art :)
                    for (final String line : TITLE) {
                        System.out.println(line);
                    }
                    final PetalsLauncher petalsLauncher = new PetalsLauncher();
                    petalsLauncher.launchPetalsServer(config);
                }
            }

        } catch (final ParseException e) {
            System.err.println("ERROR: " + e.getMessage() + "\n");
            PetalsLauncher.printUsageMessage();
            exitCode = 1;
        } catch (final Exception e) {
            System.err.println("ERROR: " + e.getMessage() + "\n");
            if (errorEnable) {
                e.printStackTrace(System.err);
            }
            exitCode = 1;
        }

        return exitCode;
    }
}
